<div class="videocontent">
<object width="400" height="330">
<param name="movie" value="http://www.youtube.com/v/{{ v }}&hl=de&fs=1"></param>
    <param name="allowFullScreen" value="true"></param>
    <param name="allowscriptaccess" value="always"></param>
    <embed src="http://www.youtube.com/v/{{ v }}&hl=de&fs=1&rel=0" type="application/x-shockwave-flash" allowscriptaccess="always" allowfullscreen="true" width="400" height="330"></embed>
</object>
</div>
