<script type="text/javascript" src="http://ajax.googleapis.com/ajax/libs/jquery/1.3.2/jquery.min.js"></script>
<script type="text/javascript" src="{{ FEINCMS_ADMIN_MEDIA }}jquery.color.js"></script>

<div id="{{ identifier }}-new">
    {{ content|safe }}
</div>

<script type="text/javascript">
opener.fe_update_content('{{ identifier }}', $('#{{ identifier }}-new').html());

window.close();
</script>
