{% extends "admin/change_form.html" %}
{% load feincms_compat_tags i18n admin_modify adminmedia %}


{% block extrahead %}{{ block.super }}
{% if FEINCMS_ADMIN_MEDIA_HOTLINKING %}
<script type="text/javascript" src="http://ajax.googleapis.com/ajax/libs/jquery/1.3.2/jquery.min.js"></script>
<script type="text/javascript" src="http://ajax.googleapis.com/ajax/libs/jqueryui/1.7.2/jquery-ui.min.js"></script>
<link type="text/css" href="http://ajax.googleapis.com/ajax/libs/jqueryui/1.7.2/themes/base/ui.all.css" rel="stylesheet" />
{% else %}
<script type="text/javascript" src="{{ FEINCMS_ADMIN_MEDIA }}jquery-1.3.2.min.js"></script>
<script type="text/javascript" src="{{ FEINCMS_ADMIN_MEDIA }}ui/jqueryui-custom-1.7.2.min.js"></script>
<link rel="stylesheet" type="text/css" href="{{ FEINCMS_ADMIN_MEDIA }}ui/jqueryui-custom-1.7.2.css" />
{% endif %}

<script type="text/javascript" src="{{ FEINCMS_ADMIN_MEDIA }}jquery.alerts.js"></script>
<script type="text/javascript" src="{{ FEINCMS_ADMIN_MEDIA }}item_editor.js"></script>

{% include "admin/feincms/_messages_js.html" %}
{% include "admin/feincms/_regions_js.html" %}

{% for inc in object.feincms_item_editor_includes.head %}{% include inc %}{% endfor %}

{% endblock %}

{% block content %}
<div class="content_main" id="frontend_editor">

<form method="post" action=".">
{% csrf_token %}
<div>
{% if is_popup %}<input type="hidden" name="_popup" value="1" />{% endif %}

    <table>
        {{ form }}
    </table>

    <div class="submit-row" >
        <input type="submit" value="{% trans "Save" %}" class="default" name="_save" />
    </div>

</div>
</form>

</div>
{% endblock %}
