{% for item in object_list %}
    <li id="item{{ item.id }}">
        <span>{{ item }}</span>
        {% if not item.is_leaf_node %}
            <ul class="ajax">
                <li>{url:?__cmd=subtree&amp;parent={{ item.id }}}</li>
            </ul>
        {% endif %}
    </li>
{% endfor %}
