<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Frameset//EN"
    "http://www.w3.org/TR/xhtml1/DTD/xhtml1-frameset.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en" dir="ltr">
<head>
<title></title>
<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
<script type="text/javascript">
if(window!=top) {
    window.location.href = '?_blank=1';
}
setInterval(function(){
    // This is really ugly. If possible this should only be done
    // when a new document has been loaded into the frame below...
    top.document.title = top.frames.frame_content.document.title;
}, 1000);
</script>
</head>
<frameset cols="250,*" rows="*" id="mainFrameset">
        <frame frameborder="0" id="frame_navigation" src="?_tree=1" name="frame_navigation" />
        <frame frameborder="0" id="frame_content" src="?_blank=1" name="frame_content" />
</frameset>
<noframes>
<body>
    <p>You need a frame-enabled browser for the split pane editor.</p>
</body>
</noframes>
</html>
