{% load i18n %}

<script type="text/javascript" charset="utf-8">
(function () {
    /* Avoid loading jQuery twice, which will break sites by resetting any pre-existing jQuery and its plugins: */
    if (typeof(jQuery) == "undefined") {
        var jQuery_scr = document.createElement("script");
        jQuery_scr.type = "text/javascript";
        jQuery_scr.src = {% if FEINCMS_ADMIN_MEDIA_HOTLINKING %}"http://ajax.googleapis.com/ajax/libs/jquery/1.3.2/jquery.min.js"{% else %}"{{ FEINCMS_ADMIN_MEDIA }}jquery-1.3.2.min.js"{% endif %};
        document.getElementsByTagName('head')[0].appendChild(jQuery_scr);
    }
})();
</script>

<script src="{{ FEINCMS_ADMIN_MEDIA }}/frontend_editing.js" type="text/javascript" charset="utf-8"></script>
<link rel="stylesheet" href="{{ FEINCMS_ADMIN_MEDIA }}/frontend_editing.css" type="text/css" media="screen" charset="utf-8">

<div id="fe_controls">
    <a href="?frontend_editing=0">{% trans "Stop Editing" %}</a>
    <a href="{% url admin:page_page_change feincms_page.pk %}">Admin Page</a>
</div>

<div id="fe_tools">
    <a id="fe_tools_edit" href="#">{% trans "edit" %}</a>
    <!--
    <a id="fe_tools_new" href="#">{% trans "new" %}</a>
    <a id="fe_tools_up" href="#">{% trans "up" %}</a>
    <a id="fe_tools_down" href="#">{% trans "down" %}</a>
    <a id="fe_tools_remove" href="#">{% trans "remove" %}</a>
    -->
</div>
