{% extends "admin/change_list.html" %}
{% load i18n admin_modify adminmedia %}

{% block extrahead %}
<script type="text/javascript" charset="utf-8">
    if (top && "frame_navigation" in top.frames) {
        // Avoid a race condition in Firefox:
        var nav_href = document.location.href.replace("_blank=1", "_tree=1");
        if (top.frames["frame_navigation"].location.href == nav_href) {
            top.frames["frame_navigation"].location.reload();
        } else {
            top.frames["frame_navigation"].location.href = nav_href;
        }
    }
</script>
{% endblock extrahead %}

{% block breadcrumbs %}
<div class="breadcrumbs">
  <a href="../../">
    {% trans "Home" %}
  </a>
   &rsaquo;
   <a href="../">
     {{ opts.app_label|capfirst }}
  </a>
  &rsaquo;
  {{ opts.verbose_name_plural|capfirst }}
</div>
{% endblock %}

{% block content %}

<div id="content-main">
    {% block object-tools %}
    {% if has_add_permission %}
    <ul class="object-tools"><li><a href="add/{% if is_popup %}?_popup=1{% endif %}" class="addlink">{% blocktrans with opts.verbose_name as name %}Add {{ name }}{% endblocktrans %}</a></li></ul>
    {% endif %}
    {% endblock %}
</div>

<p>{% trans "Select an item on the left side if you want to edit it." %}</p>
<p>{% trans "You can change the structure of the tree by drag-dropping elements. Please note that changes will be saved immediately. " %}</p>
<p>{% trans "The context menu on the tree root and tree nodes provide you with additional modes of operation." %}</p>

{% endblock %}
