.simpleTree
{

    margin:0;
    padding:0;
    /*
    overflow:auto;
    width: 250px;
    height:350px;
    overflow:auto;
    border: 1px solid #444444;
    */
}
.simpleTree li
{
    list-style: none;
    margin:0;
    padding:0 0 0 34px;
    line-height: 14px;
}
.simpleTree li span
{
    display:inline;
    clear: left;
    white-space: nowrap;
}
.simpleTree ul
{
    margin:0;
    padding:0;
}
.simpleTree .root
{
    margin-left:-16px;
    background: url(images/root.gif) no-repeat 16px 0 #ffffff;
}
.simpleTree .line
{
    margin:0 0 0 -16px;
    padding:0;
    line-height: 3px;
    height:3px;
    font-size:3px;
    background: url(images/line_bg.gif) 0 0 no-repeat transparent;
}
.simpleTree .line-last
{
    margin:0 0 0 -16px;
    padding:0;
    line-height: 3px;
    height:3px;
    font-size:3px;
    background: url(images/spacer.gif) 0 0 no-repeat transparent;
}
.simpleTree .line-over
{
    margin:0 0 0 -16px;
    padding:0;
    line-height: 3px;
    height:3px;
    font-size:3px;
    background: url(images/line_bg_over.gif) 0 0 no-repeat transparent;
}
.simpleTree .line-over-last
{
    margin:0 0 0 -16px;
    padding:0;
    line-height: 3px;
    height:3px;
    font-size:3px;
    background: url(images/line_bg_over_last.gif) 0 0 no-repeat transparent;
}
.simpleTree .folder-open
{
    margin-left:-16px;
    background: url(images/collapsable.gif) 0 -2px no-repeat #fff;
}
.simpleTree .folder-open-last
{
    margin-left:-16px;
    background: url(images/collapsable-last.gif) 0 -2px no-repeat #fff;
}
.simpleTree .folder-close
{
    margin-left:-16px;
    background: url(images/expandable.gif) 0 -2px no-repeat #fff;
}
.simpleTree .folder-close-last
{
    margin-left:-16px;
    background: url(images/expandable-last.gif) 0 -2px no-repeat #fff;
}
.simpleTree .doc
{
    margin-left:-16px;
    background: url(images/leaf.gif) 0 -1px no-repeat #fff;
}
.simpleTree .doc-last
{
    margin-left:-16px;
    background: url(images/leaf-last.gif) 0 -1px no-repeat #fff;
}
.simpleTree .ajax
{
    background: url(images/spinner.gif) no-repeat 0 0 #ffffff;
    height: 16px;
    display:none;
}
.simpleTree .ajax li
{
    display:none;
    margin:0;
    padding:0;
}
.simpleTree .trigger
{
    display:inline;
    margin-left:-32px;
    width: 28px;
    height: 11px;
    cursor:pointer;
}
.simpleTree .text
{
    cursor: default;
}
.simpleTree .active
{
    cursor: default;
    background-color:#F7BE77;
    padding:0px 2px;
    border: 1px dashed #444;
}
#drag_container
{
    background:#ffffff;
    color:#000;
    font: normal 11px arial, tahoma, helvetica, sans-serif;
    border: 1px dashed #767676;
}
#drag_container ul
{
    list-style: none;
    padding:0;
    margin:0;
}

#drag_container li
{
    list-style: none;
    background-color:#ffffff;
    line-height:18px;
    white-space: nowrap;
    padding:1px 1px 0px 16px;
    margin:0;
}
#drag_container li span
{
    padding:0;
}

#drag_container li.doc, #drag_container li.doc-last
{
    background: url(images/leaf.gif) no-repeat -17px 0 #ffffff;
}
#drag_container .folder-close, #drag_container .folder-close-last
{
    background: url(images/expandable.gif) no-repeat -17px 0 #ffffff;
}

#drag_container .folder-open, #drag_container .folder-open-last
{
    background: url(images/collapsable.gif) no-repeat -17px 0 #ffffff;
}
.contextMenu
{
    display:none;
}
