<div class="image">
    <img src="{{ content.mediafile.file.url }}" alt="{{ content.mediafile.translation.caption }}" />
    <span class="caption">{{ content.mediafile.translation.caption }}</span>
    {% if content.mediafile.copyright %}<span class="copyright">{{ content.mediafile.copyright }}</span>{% endif %}
</div>
